/**
 * @file
 * VuoList_VuoDistribution3d C type definition.
 *
 * @copyright Copyright © 2012–2017 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoDistribution3d VuoList_VuoDistribution3d
 * A list of @ref VuoDistribution3d elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoDistribution3d_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoDistribution3d elements.
 */
typedef const struct VuoList_VuoDistribution3d_struct { void *l; } * VuoList_VuoDistribution3d;
/// @}
#endif

/**
 * Creates a new list of @ref VuoDistribution3d elements.
 */
VuoList_VuoDistribution3d VuoListCreate_VuoDistribution3d(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoDistribution3d to quickly initialize a large list.
 */
VuoList_VuoDistribution3d VuoListCreateWithCount_VuoDistribution3d(const unsigned long count, const VuoDistribution3d value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoDistribution3d VuoListCopy_VuoDistribution3d(const VuoList_VuoDistribution3d list);

/**
 * Returns the @ref VuoDistribution3d at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoDistribution3d VuoListGetValue_VuoDistribution3d(const VuoList_VuoDistribution3d list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoDistribution3d *VuoListGetData_VuoDistribution3d(const VuoList_VuoDistribution3d list);

/**
 * Changes the @ref VuoDistribution3d at @c index.
 * Index values start at 1.
 *
 * If `expandListIfNeeded` is false: If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 *
 * If `expandListIfNeeded` is true: If the list has less than `index` elements, it is expanded to accommodate the specified `index`.
 * If `index` is 0, the list is resized to contain 1 item set to `value`.
 */
void VuoListSetValue_VuoDistribution3d(const VuoList_VuoDistribution3d list, const VuoDistribution3d value, const unsigned long index, bool expandListIfNeeded);

/**
 * Inserts the `VuoDistribution3d` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoDistribution3d(const VuoList_VuoDistribution3d list, const VuoDistribution3d value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoDistribution3d(VuoList_VuoDistribution3d list, const VuoDistribution3d value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoDistribution3d(VuoList_VuoDistribution3d list, const VuoDistribution3d value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoDistribution3d(VuoList_VuoDistribution3d list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoDistribution3d_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoDistribution3d(VuoList_VuoDistribution3d list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoDistribution3d_areEqual(const VuoList_VuoDistribution3d a, const VuoList_VuoDistribution3d b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoDistribution3d_isLessThan(const VuoList_VuoDistribution3d a, const VuoList_VuoDistribution3d b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoDistribution3d(VuoList_VuoDistribution3d list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoDistribution3d(VuoList_VuoDistribution3d list);

/**
 * Returns a new list containing items from the original list in the range specified by `startIndex` and `itemCount`.
 *
 * Items in the new list are retained (not copied) from the original list.
 */
VuoList_VuoDistribution3d VuoListSubset_VuoDistribution3d(VuoList_VuoDistribution3d list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoDistribution3d(VuoList_VuoDistribution3d list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoDistribution3d(VuoList_VuoDistribution3d list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoDistribution3d(VuoList_VuoDistribution3d list);

/**
 * Removes the `VuoDistribution3d` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoDistribution3d(VuoList_VuoDistribution3d list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoDistribution3d(const VuoList_VuoDistribution3d list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoDistribution3d VuoList_VuoDistribution3d_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoDistribution3d_getJson(const VuoList_VuoDistribution3d value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoDistribution3d_getSummary(const VuoList_VuoDistribution3d value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoDistribution3d VuoList_VuoDistribution3d_makeFromString(const char *str);
char * VuoList_VuoDistribution3d_getString(const VuoList_VuoDistribution3d value);
/// @}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
